#!/usr/bin/python3
# Simple script for converting darktable .dtstyle files to .xmp (version 2)

from sys import argv
from os import path
import xml.etree.ElementTree as ET

if len(argv) != 3 :
    print ("usage:",argv[0],"style.dtstyle file.xmp")
    exit(1)

# Check if the input file exists.
if not path.exists(argv[1]):
    print ("ERROR: input file:",argv[1],"doesn't exists.")
    exit(1)

# Check if the requested output file already exist
if path.exists(argv[2]):
    print ("ERROR: output file:",argv[2],"already exists.")
    exit(1)

try:
    styletree = ET.parse(argv[1]) or die("This doesn't work for me")
except ET.ParseError:
    print ("ERROR: input file:",argv[1],"is not a valid dtsyle-file.")
    exit(1)

styleroot = styletree.getroot()

# Create a new xml structure.
xmpmeta=ET.Element("x:xmpmeta",{"xmlns:x":"adobe:ns:meta/",
                                "x:xmptk":"XMP Core 4.4.0-Exiv2"})
rdf=ET.SubElement(xmpmeta,"rdf:RDF",{"xmlns:rdf":"http://www.w3.org/1999/02/22-rdf-syntax-ns#"})
description=ET.SubElement(rdf,"rdf:Description",{"rdf:about":"",
                              "xmlns:xmp":"http://ns.adobe.com/xap/1.0/",
                              "xmlns:xmpMM":"http://ns.adobe.com/xap/1.0/mm/",
                              "xmlns:dc":"http://purl.org/dc/elements/1.1/",
                              "xmlns:darktable":"http://darktable.sf.net/",
                              "xmp:Rating":"0",
                              "xmpMM:DerivedFrom":"PureAwesome.raw",
                              "darktable:xmp_version":"2",
                              "darktable:raw_params":"0",
                              "darktable:auto_presets_applied":"1"})
# darktable:history_end is not needed with a conversion, darktable will defaults to the topmost element in the history stack
# "darktable:history_end":"8"})
maskid=ET.SubElement(description,"darktable:mask_id")
maskid.append(ET.Element("rdf:Seq"))
masktype=ET.SubElement(description,"darktable:mask_type")
masktype.append(ET.Element("rdf:Seq"))
maskname=ET.SubElement(description,"darktable:mask_name")
maskname.append(ET.Element("rdf:Seq"))
maskversion=ET.SubElement(description,"darktable:mask_version")
maskversion.append(ET.Element("rdf:Seq"))
mask=ET.SubElement(description,"darktable:mask")
mask.append(ET.Element("rdf:Seq"))
masknb=ET.SubElement(description,"darktable:mask_nb")
masknb.append(ET.Element("rdf:Seq"))
masksrc=ET.SubElement(description,"darktable:mask_src")
masksrc.append(ET.Element("rdf:Seq"))
darktablehistory=ET.SubElement(description,"darktable:history")
darktablehistoryid=ET.SubElement(darktablehistory,"rdf:Seq")

# iterate through the dtstyle to find and add the used plugins to the xmp
for plugins in styleroot.findall('./style/plugin'):
    enabled = plugins.find('enabled')
    if enabled.text == "1":
        li=ET.SubElement(darktablehistoryid,"rdf:li")
        li.set("darktable:enabled","1")

        modversion = plugins.find('module')
        if modversion != None:
            li.set("darktable:modversion",modversion.text)

        operation = plugins.find('operation')
        if operation != None:
            li.set("darktable:operation",operation.text)

        params = plugins.find('op_params')
        if params != None:
            li.set("darktable:params",params.text)

        blendop_params = plugins.find('blendop_params')
        if blendop_params != None:
            li.set("darktable:blendop_params",blendop_params.text)

        blendop_version = plugins.find('blendop_version')
        if blendop_version != None:
            li.set("darktable:blendop_version",blendop_version.text)

        multi_name = plugins.find('multi_name')
        if multi_name != None:
            if multi_name.text is None:
                li.set("darktable:multi_name","")
            else:
                li.set("darktable:multi_name",multi_name.text)

        multi_priority = plugins.find('multi_priority')
        if multi_priority != None:
            li.set("darktable:multi_priority",multi_priority.text)

# Write the newly created xmp to a file
ET.ElementTree(xmpmeta).write(argv[2],"UTF-8")



#
#   This file is part of darktable,
#   Copyright (C) 2022 darktable developers.
#
#   darktable is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   darktable is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with darktable.  If not, see <http://www.gnu.org/licenses/>.
#

# Read each IOP file, parse it for buffer allocs, and check if we have a corresponding free

import os
import re
directory = "../src/iop/"

alloc_regex = r"([a-zA-Z0-9_\-\>\.\[\]]+) = (dt_|c|m|dt_opencl_)alloc.*\(.+\)"


def find_call_and_line(elem, file, directory):
  line_number = 0
  f = open(os.path.join(directory, file), "r")

  # find the line(s) in file
  for line in f:
    line_number += 1
    if elem in line:
      print("\t\t line", line_number, ":", elem.strip())

  f.close()

for file in sorted(os.listdir(directory)):
  if file.endswith(".c") or file.endswith(".h"):
    f = open(os.path.join(directory, file), "r")
    content = f.read()

    # Find all allocs and extract the name of the pointer to the allocation
    matches = re.finditer(alloc_regex, content, re.MULTILINE)

    safe_allocs = 0
    faulty_allocs = 0
    suspicious_allocs = 0

    print("%s" % file)

    for matchNum, match in enumerate(matches):
      # Get the name of the pointer to the allocation
      variable_name = match.group(1)
      alloc_type = match.group(2)

      # Look how many times the variable is allocated
      variable_alloc_regex = " %s = %salloc.*\(.+\)" % (variable_name, alloc_type)
      matches2 = re.findall(variable_alloc_regex, content, re.MULTILINE)
      allocs = len(matches2)
      matches2 = set(matches2)

      # Look how many times the variable is freed
      variable_free_regex = r""
      buffer_type = ""
      if(alloc_type == "dt_opencl_"):
        variable_free_regex  = ".*release_mem_object.*\(%s\)" % variable_name
        buffer_type = "OpenCL"
      else:
        variable_free_regex  = " \S*free.*\(%s\)" % variable_name
        buffer_type = "C"
      matches3 = re.findall(variable_free_regex, content, re.MULTILINE)
      frees = len(matches3)
      matches3 = set(matches3)

      # Note that for OpenCL, we may have more than one free for each alloc because of the error go-to
      if(frees < allocs and frees == 0):
        print("\tERROR: %s buffer `%s` is allocated %i time(s) but never freed" % (buffer_type, variable_name, allocs))
        for elem in matches2:
          find_call_and_line(elem, file, directory)
        for elem in matches3:
          find_call_and_line(elem, file, directory)

        faulty_allocs += 1

      elif(frees < allocs and frees > 0):
        print("\tWARNING: %s buffer `%s` is allocated %i time(s) but freed %i time(s)" % (buffer_type, variable_name, allocs, frees))
        for elem in matches2:
          find_call_and_line(elem, file, directory)
        for elem in matches3:
          find_call_and_line(elem, file, directory)

        suspicious_allocs += 1

      else:
        safe_allocs += 1

    msg_type = "INFO"
    if(suspicious_allocs > 0):
      msg_type = "WARNING"
    if(faulty_allocs > 0):
      msg_type = "ERROR"

    print("\t%s: %i safe alloc(s) detected over %i\n" % (msg_type, safe_allocs, safe_allocs + faulty_allocs + suspicious_allocs))

    f.close()



import importlib
import logging
import pkgutil
from litsupport.modules import modules

# Load our custom modules
for importer, modname, ispkg in pkgutil.walk_packages(path=__path__,
                                                      prefix=__name__ + '.'):
    module = importlib.import_module(modname)
    if not hasattr(module, 'mutatePlan'):
        logging.error('Skipping %s: No mutatePlan function' % modname)
        continue
    assert modname.startswith('litsupport_rawspeed.modules.')
    shortname = modname[len('litsupport_rawspeed.modules.'):]
    modules[shortname] = module
    logging.info("Loaded test module %s" % module.__file__)
